markup.push({
     
    "InspectorButton": {                // so the intrinsic size of the buttons is set in one place
        nsClass: "FGButtonView",
        gxIntrinsicHeight: 24,
        gxIntrinsicWidth: 24,
    },

    "InspectorLabel": {
        "+": "LabelView",
        gxValueKeyPath: "",
    },   

	"ColorSliderMenuItem" : {
		"+" : "ColorSlider",
        gxKnobPainter: {
			"+" : "ColorSliderKnob",
			gxEdgeInsets: "-0.5,0.5,0.5,-0.5",
		},
	},
			
	"ColorSliderKnob" : {
		nsClass: "GXRoundRectPainter",
		gxDefaultSize: "0,0",
		gxEdgeInsets: "-0.5,-0.5,0.5,0.5",
		gxCornerRadius: 9,
		gxFrameColor: Colors.tertiaryContent,
		gxStrokeWidth: 2,
		gxShadow: "{0.0, 0.0, 0.0 0.7}, {0.0, -0.5}, 2.5",
		"disabled": {
			gxFrameColor: Colors.tertiaryContent,
			gxEdgeInsets: "999,999,999,999",    // hackish, force empty rect to avoid drawing
		},
		"active": {
			// prevent drawing using zero size when disabled
			gxFrameColor: Colors.tileTint,
			gxDefaultSize: "19,19",
		}
	}, 
 
}); 

function SecuritySection()
{
    return VStack([ 
        Padding(),
        HStack([
            Padding(4),
            Label("Security").fontSize(14).fontWeight(0.4),
            Spacer()
        ]), 
        Padding(),
        HStack([
            Padding(4),
            SwitchSection("Lock This Set", "tileSet.security.setLocked"), 
            Padding(4),
        ]),
        Padding(),
        HStack([
            Padding(4),
            SwitchSection("Require Password to Unlock", "tileSet.security.setPasswordRequired"), 
            Padding(4),
        ]),
        Padding(10),
        VStack([
            PushButton("Change Password...") 
                .keyPath("tileSet.security.setNewPassword")
                .width(150),
            Padding(6),
        ])
        .align("left")
        .makeView()
        .expandKey("tileSet.security.hasPassword")
        .viewName("changePasswordButtonContainer")
        .fillWidth(), 
    ]).makeView().fillWidth().leftPad(11);
}

function ControllersSection()
{
    return VList([  
        // OSC Section 
        VStack([
            Padding(9),    
            HStack([
                Padding(2),
                Label("OSC").fontSize(13), 
                Padding(5),
                ZStack([ 
                    TextFieldView()  
                            .keyPath("oscAddress")
                            .setAXTitle("OSC Address")
                            .setAXRole("The address the selected tiles use to control playback.")
                            .backPainter(null) 
                            .leftPad(2)
                            .rightPad(30)
                            .height(28)
                            .setValue(true,"gxIsSingleLine", true) 
                            .setValue(false,"gxEditable", true)
                            .setValue(true, "gxSelectable", true)
                            .painterByMutating("gxTextPainter", (painter) => { 
                                return painter.lineBreakMode(2);
                            })
                            , 
        
                    PopupButton("FGPopupButton", true)
                        .backPainter(null)
                        .keyPath("visibleOSCAction")
                        .menuItemsPath("oscActions")
                        .centerV()
                        .fillHeight() 
                        .height(28)
                        .rightPad(0)
                        .width(22),
                ])
                .makeView()
                .backPainter(TextFieldBack())
            ])
            .makeView()
            
            .fillWidth(), 
        ]) 
        .makeView() 
        .addLayout(`V:[self(>=${Constants.footerHeight - 11})]`)
        .showKey("showOSCSection"),

        View().height(6).hideKey("hideControllerDivision"),  

        // MIDI Section
        HStack([
            Padding(2),
            Label("MIDI").fontSize(13), 
            ZStack([   
                Label("MIDI Note Label")
                    .textAlign(0,0.5)
                    .fontColor(Colors.secondaryContent) 
                    .fontSize(13)
                    .intrinsicSize(-1,-1)
                    .fillHeight()
                    .leftPad(40)    
                    .rightPad(60)
                    .setAXTitle("Assigned MIDI Note")
                    .hideKey("midiResponder.learningMidiEvent")
                    .keyPath("midiResponder.midiNoteText"), 
                PushButton("Learn")
                    .toolTipKeyPath("midiResponder.midiLearnToolTip") 
                    .keyPath("midiResponder.learnMidi")
                    .setAXTitle("Learn MIDI")
                    .hideKey(null)
                    .centerV()
                    .width(60)
                    .rightPad(0),
                PushButton("Clear") 
                    .toolTipKeyPath("midiResponder.midiLearnToolTip") 
                    .width(60)
                    .centerV()
                    .keyPath("midiResponder.clearMidi")
                    .setAXTitle("Clear MIDI")
                    .showKey("midiResponder.showMidiClear")
                    .rightPad(0),
                PushButton("Learning...") 
                    .toolTipKeyPath("midiResponder.midiLearnToolTip") 
                    .keyPath("midiResponder.learnMidi") 
                    .width(90)
                    .centerV()
                    .showKey("midiResponder.learningMidiEvent")
                    .rightPad(0),
            ]),
        ]) 
        .makeView() 
        .addLayout(`V:[self(>=${Constants.footerHeight - 11})]`)
        .keyPath("")
        .disableKey("locked")
        .fillWidth()
        .showKey("midiResponder.showMidiSection"),  

        View().height(4),

        View().height(3).showKey("hideControllerDivision"), 
    ]) 
    .fillWidth()
    .showKey("showControllersSection") 
    .padding(12,12,0,0);
}

function AutomationSection(listName)
{   
    return VList([  
        
        Divider().viewName("headerView"),
        
        HStack([
            Padding(15),
            Label("Automation").ignoreAX().fontSize(14).fontWeight(0.4),
            Padding(),
            Button(Constants.warningEmoji) 
                .toolTip("Automation permissions required")
                .backPainter(null)
                .size(14)
                .keyPath("showNeedsPermissions")
                .showKey("showNeedsPermissions"),
            Spacer()
        ]),
        
        VList([ 
            // New Automation Button 
            HStack([
                Padding(13),
                ZStack([
                    Button() 
                        .height(22)
                        .keyPath("addAutomation")
                        .setAXTitle("New Automation")
                        .fillWidth()
                        .backPainter(TemplatePainter("SetButtonBack.setsPlus").align(0,0.5))
                        .toolTip("Add new automation"),
                    Label("New Automation")
                        .fontSize(13)
                        .textAlign(1,0.5)
                        .height(22)
                        .disableKey("locked")
                        .ignoreAX()
                        .fillFrame(),
                ]) 
                .makeView() 
                .width(130), 
                
                Spacer(),

                Button()
                    .size(20)
                    .backPainter(TemplatePainter("SetButtonBack.setsMinus").align(0,0.5))
                    .toolTip("Delete selected automation")
                    .keyPath("deleteSelectedAutomation")
                    .setAXTitle("Delete Selected Automation")
                    .viewName("deleteSetButton"),
                Padding(15),
            ])
            .makeView()
            .viewName("footerView")
            .fillWidth()
            .intrinsicSize(-1,22)
        ])
        .rowSpacing(10)
        // .hPad(13)
        .setAXRole("Automation List")
        .viewName(listName),   
    ])
    .makeView()
    .hideKey("automationsHidden") 
    .keyPath("")
    .topPad(10)
    .rowSpacing(7)
    .fillWidth();
}

function InspectorPanel()
{
    return ZStack([ 
        ScrollView(
            TileInspectorPanel()
        )
        .hScroller(null)
        .fillFrame()
        .backColor(Colors.mainBackground)
        .keyPath("")  
        .ignoreAX()
        .setValue(true, "gxOverlaysScroller")
        .viewName("tileInspectorScrollView"),
       
        ScrollView(
            SetInspectorPanel()  
        )
        .hScroller(null)
        .backColor(Colors.mainBackground)
        .keyPath("")
        .fillFrame()
        .setValue(true, "gxOverlaysScroller")
        .viewName("setInspectorScrollView")
    ])
    .makeView("FGPanelBackgroundView")  
    .backPainter(Painter().fill(Colors.mainBackground))
    .keyPath("")   
    .setValues({ 
        gxAlwaysUpdate: true,
        gxMinimumPanelSize: Constants.minInspectorWidth,
        gxFrameRect: `0,0,${Constants.minInspectorWidth},0`,
        gxHoldingPriority: 440, 
        gxCollapsible: true,
        gxViewName: "inspectorPanel"
    })
    .addLayout(`H:[self(>=${Constants.minInspectorWidth})]`);
}

function SwitchSection(title,keypath)
{
    return  HStack([
        Label(title)
            .fontSize(13)
            .ignoreAX()
            .fontColor(Colors.fontColor),
        Spacer(),
        View("GXSwitchView") 
            .keyPath(keypath)
            .size(40,20)
            .backPainter(Painter()
                .radius(-1)
                .fill(Colors.secondaryContent.addStyle("on", Colors.tileTint))
                .addStyleWithPainter("increasedContrast", (painter) => {
                    return painter.stroke(1, Colors.contrastBorder)
                })
            )
            .setAXTitle(title === "" ? null : title)
            .setValues({
                gxThumbPainter : Painter()
                    .size(20)
                    .radius(-1)
                    .inset(2)
                    .fill(Colors.tileTintContent.showPressed())
                    .shadow()
            })
    ])
}