   
function PredicatePopup()
{
    return PopupButton(null,false,22).height(30);
}

function PredicateTextField()
{
    return TextFieldView("FGRuleEditorTextField")
        .formatter(Formatter("").className("GXTextPredicateFormatter"))
        .painterByMutating("gxBackPainter", painter=>{
            return painter.insets(3,0,3,0)
        })
        .insets(3,0,0,0)
        .height(26)
        .width(200);
}

function PredicateNumberTextField()
{
    return PredicateTextField().formatter(NumberFormatter());
}

function AddRowButton()
{
    return Button() 
        .size(22)
        .action("addRow:") 
        .viewName("addButton")
        .setAXTitle("New Filter")
        .fillWidth()
        .backPainter(TemplatePainter("SetButtonBack.setsPlus").align(0,0.5))
        .toolTip("Add new filter");
}

function DeleteRowButton()
{
    return Button() 
        .size(22)
        .action("deleteRow:")
        .viewName("deleteButton")
        .setAXTitle("Delete Filter") 
        .backPainter(TemplatePainter("SetButtonBack.setsMinus").align(0,0.5))
        .toolTip("Add new filter");
}

function FirstPredicatePopup()
{
    return PredicatePopup().menuItems([
        MenuItem("Tile", "Tile"),
        MenuItem("Tile Notes", "tileNotes"),
        MenuItem("Tile Output", "outputDeviceReference"),
//        MenuItem("Tile Emoji", "tileIcon"),
        MenuItem("Set", "Set"),
        MenuItem("Set Notes", "setNotes"),
        MenuItem("Title", "title"),
        MenuItem("Color", "colorIndex"),
        MenuItem("Duration", "duration"),
        MenuItem("Date Played", "lastPlayStart"),
        MenuItem("Play Count", "playCount"),
    ])
    .width(180)
    .keyPath("firstValue");
}

function AndOrPopup()
{
    return PredicatePopup().menuItems([
        MenuItem("is", "==[cd]"),
        MenuItem("is not", "!=[cd]"),
    ])
    .width(100)
    .keyPath("secondValue");
}

function StringComparePopup()
{
    return PredicatePopup().menuItems([
        MenuItem("contains", "CONTAINS[cd]"),
        MenuItem("is", "==[cd]"),
        MenuItem("is not", "!=[cd]"), 
        MenuItem("begins with", "BEGINSWITH[cd]"),
        MenuItem("ends with", "ENDSWITH[cd]"),
    ])
    .width(100)
    .keyPath("secondValue");
}

function NumberComparePopup()
{
    return PredicatePopup().menuItems([
        MenuItem("is equal to", "==[cd]"),
        MenuItem("is not equal to", "!=[cd]"), 
        MenuItem("is less than", "<[cd]"),
        MenuItem("is greater than", ">[cd]"),
        MenuItem("is less than or equal to", "<=[cd]"),
        MenuItem("is greater than or equal to", ">=[cd]"),
    ]) 
    .keyPath("secondValue");
}

function DurationComparePopup()
{
    return PredicatePopup().menuItems([
        MenuItem("is greater than", ">[cd]"), 
        MenuItem("is greater than or equal to", ">=[cd]"),
        MenuItem("is less than", "<[cd]"), 
        MenuItem("is less than or equal to", "<=[cd]"),
    ]) 
    .keyPath("secondValue");
}

function DateComparePopup()
{
    return PredicatePopup().menuItems([
        MenuItem("is", "==[cd]"),
        MenuItem("is not", "!=[cd]"), 
        MenuItem("is before", "<[cd]"),
        MenuItem("is after", ">[cd]"),
        MenuItem("is on or before", "<=[cd]"),
        MenuItem("is on or after", ">=[cd]"),
        MenuItem("in last", "inLastMinute:").showKey("showMinutes"),
        MenuItem("in last", "inLastHour:").showKey("showHours"),
        MenuItem("in last", "inLastDay:").showKey("showDays"),
        MenuItem("in last", "inLastWeek:").showKey("showWeeks"),
        MenuItem("in last", "inLastMonth:").showKey("showMonths"),
        MenuItem("in last", "inLastYear:").showKey("showYears"),
    ]) 
    .keyPath("secondValue");
}

function CompoundPredicate()
{
    return VStack([ 
        HStack([
            Padding(11),  
            PredicatePopup().menuItems([
                MenuItem("All", 1),
                MenuItem("Any", 2), 
                MenuItem("None", 0),
            ])
                .keyPath("predicateType")
                .setAXTitle("of the folling are true:")
                .toolTip("Filter type")
                .width(80),
            Padding(5),
            Label("of the following are true:").fontSize(13),
            Spacer(), 
            Padding(8),
        ]), 
        Padding(2),
        Divider(),
    ])
    .makeView()
    .fillWidth() 
    .keyPath("");
} 

function AddDeleteButtons()
{
    return HStack([  
        DeleteRowButton(),
        Padding(5),
        AddRowButton(),
        Padding(8),
    ]);
}

function StandardPredicate(firstView,secondView)
{
    return VStack([
        Padding(2),
        HStack([
            Padding(31), 
    
            FirstPredicatePopup()
                .setAXRole("Filter")
                .toolTip("Filter"),
            
            Padding(7),
            
            firstView.setAXRole("Filter operator")
                .toolTip("Filter operator")
                .width(185)
                .keyPath("secondValue"),
            
            Padding(7),
    
            secondView
                .setAXRole("Filter value")
                .toolTip("Filter value")
                .keyPath("thirdValue"),
            
            Spacer(), 
            AddDeleteButtons(),  
        ]),
        Padding(2),
        Divider().viewName("divider"),
    ])
    .makeView()
    .setAXRole("Filter group")
    .setAXHelpPath("axHelp") 
    .fillWidth() 
    .keyPath("");
}

function TilePopupMenu()
{
    return PredicatePopup().menuItems([
        MenuItem("favorited", "ruleFavorite"),
        MenuItem("looping", "ruleLoop"),
        MenuItem("soloed", "ruleSolo"),
        MenuItem("Pausable", "ruleResume"),
        MenuItem("hold to play", "ruleHoldToPlay"),
    ]);
}

function TilePredicate()
{
    return StandardPredicate(AndOrPopup(),  TilePopupMenu().width(200));
}

function StringPredicate()
{
    return StandardPredicate(StringComparePopup(), PredicateTextField());
}

function SetPredicate()
{
    return StandardPredicate(AndOrPopup(), PredicatePopup()
        .width(200)
        .menuItemsPath("setItems"));
}

function OutputDevicePredicate()
{
    return StandardPredicate(AndOrPopup(), PredicatePopup()
        .width(200)
        .menuItemsPath("outputItems"));
}

function PredicateTimePicker()
{
    return View("FGTimePickerWrapped").width(197).height(26)
} 

function ColorPickerPredicate()
{ 
    return StandardPredicate(AndOrPopup(), ColorPickerView());
}

function LastXTimePredicate()
{
    return StandardPredicate(DateComparePopup(), HStack([
        PredicateNumberTextField().keyPath("thirdValue").width(95),
        Padding(5),
        PredicatePopup().menuItemsPath("dateItems").keyPath("secondValue").width(100),
    ])
    .makeView());
}

function RuleEditor()
{
    return View("FGRuleEditor")
    .viewName("ruleEditor") 
    .setValues({
        "gxSupportedPredicates" : [
            "colorIndex ==[cd] 0", // By being first, this will be the default predicate displayed when the rule editor is first opened.
            "Tile ==[cd] ruleFavorite",
            "tileNotes ==[cd] \"\"",
            "outputDeviceReference ==[cd] \"\"", 
            "Set ==[cd] \"\"",
            "setNotes ==[cd] \"\"",
            "title ==[cd] \"\"", 
            "duration >[cd] 0",
            "lastPlayStart >=[cd] CAST(0.0, \"NSDate\")",
            "playCount ==[cd] 0",
        ],
        "gxDefaultValues" : {
            "lastPlayStart" : "CAST(0.0, \"NSDate\")",
            "inLastMinute:" : "0",
            "inLastHour:"   : "0",
            "inLastDay:"    : "0",
            "inLastWeek:"   : "0",
            "inLastMonth:"  : "0",
            "inLastYear:"   : "0",
        }  
    });
}

markup.push({
    "FALSEPREDICATEPredicateView"              : CompoundPredicate(),
    "TRUEPREDICATEPredicateView"               : CompoundPredicate(),
    "NOTPredicateView"                         : CompoundPredicate(), 
    "TilePredicateView"                        : TilePredicate(),
    "titlePredicateView"                       : StringPredicate(),
    "tileNotesPredicateView"                   : StringPredicate(),
    "tileIconPredicateView"                    : StringPredicate(),
    "outputDeviceReferencePredicateView"       : OutputDevicePredicate(),
    "setNotesPredicateView"                    : StringPredicate(),
    "SetPredicateView"                         : SetPredicate(),
    "colorIndexPredicateView"                  : ColorPickerPredicate(),
    "playCountPredicateView"                   : StandardPredicate(NumberComparePopup(), PredicateNumberTextField().width(200)),
    "lastPlayStartPredicateView"               : StandardPredicate(DateComparePopup(), View("FGDatePicker").width(195).height(26)),
    "lastPlayStartinLastMinute:PredicateView"  : LastXTimePredicate(),
    "lastPlayStartinLastHour:PredicateView"    : LastXTimePredicate(),
    "lastPlayStartinLastWeek:PredicateView"    : LastXTimePredicate(),
    "lastPlayStartinLastDay:PredicateView"     : LastXTimePredicate(),
    "lastPlayStartinLastMonth:PredicateView"   : LastXTimePredicate(),
    "lastPlayStartinLastYear:PredicateView"    : LastXTimePredicate(),
    "durationPredicateView"                    : StandardPredicate(DurationComparePopup(), PredicateTimePicker()),
});
