function TextFieldBack()
{
	return Painter() 
		.stroke(1, Colors.secondaryBackgroundMix.addStyle("increasedContrast", Colors.contrastBorder))
		.fill(MainThemeColor(Colors.white, Colors.secondaryBackground))
		.inset(1)
		.radius(8);
}

function TextFieldView(className = "GXTextField")
{
	return View(className)   
		.fillHeight() 
		.setValues({
			gxEdgeInsets: "5,5,5,5",
			gxSelectionColor: Colors.tileTint.withAlpha(0.25), 
			gxIsSingleLine: false,
			gxPasswordAutofillEnabled: false
		})
		.backPainter(TextFieldBack())
		.textPainter(LabelPainter()
			.lineBreakMode(0)
			.backColor(Colors.invisible)
			.align(0,0)
			.fontSize(13)
			.color(MainThemeColor(Colors.black, Colors.white).addStyle("locked", Colors.tertiary)))
}
 
function FramedTextField(labelText = "", keyPath, labelAlign="alignCenterY", className = "GXTextField")
{    
	return HStack([
		TextFieldView(className)  
				.inset(0)
				.viewName("textField")
				.keyPath(keyPath)
				.setAXTitle(labelText)  
				.setValue(Colors.tileTint.withAlpha(0.3), "gxSelectionColor")
				.label(labelText,labelText == "" ? 0 : 5, labelText == "" ? 0 : 5,labelAlign) 
	])
	.makeView()
	.height(26)
} 

function ReadoutLabel(text)
{
	var label = Label(text)
	.textAlign(1,0.5)
	.fontColor(Colors.fontColor.showDisabled())
	.fontSize(13)
	label.gxIntrinsicHeight = null
	label.gxIntrinsicWidth = null
	return label;
}  

function PageButton(prev)
{
	return Button(prev ? "‹" : "›")
		.forePainter(LabelPainter().color(Colors.tileTintPrimary.showDisabled(Colors.black)).fontSize(16).offset((prev ? -0.5 : 0.5),-0.5))
		.backPainter(Painter().fill(MainThemeColor(colorRGBA(230,211,255,100), colorRGBA(89,74,106,100)).showPressed()).radius(-1)).size(15);
}
 
markup.push({
    "ESCKeyProtectorView" : VStack([
		Label("Hold '<bold>Esc</bold>' to Stop All Playback") 
			.textAlign(0.5,0.5)
			.useInlineMarkup()  
			.fontSize(20)
			.fontWeight(-0.4)
			.intrinsicSize(-1,null)
			.fontColor(Colors.white) 
			.fillFrame()
			
	])
	.makeView()
	.width(320)
	.height(60)
	.setAXTitle("Hold Esc to Stop All Playback")
	.setAXRole("Escape Key Protector")
	.backPainter(Painter().fill(Colors.pitchBlack.withAlpha(0.5)).radius(30).stroke(1, Colors.white.withAlpha(0.25)))
	// .shadow()
	.center(),
	
	"prevButton" : PageButton(true),
	"nextButton" : PageButton(false),

	"ColorPicker" : ColorPickerView(),
});

function ColorPickerView()
{
    function CircleColor(color)
    {
        var colorKeys = {
            "0": "purple",
            "1": "pink",
            "2": "red",
            "3": "orange",
            "4": "yellow",
            "5": "green",
            "6": "light blue",
            "7": "blue",
            "8": "white",
        }
        let tileColor = MainThemeColor(Colors.tileTint.lightMode[color], Colors.tileTint.darkMode[color])
        return ZStack([
            Button()
				// .setValue(true, "gxOperateOnMouseDown")
                .checkBoxRole()
                .setAXTitle(colorKeys[color]) 
                .keyPath("set" + color)
                .forePainter(null)
                .backPainter(Painter().fill(tileColor.showPressed()).radius(-1)).size(12).center(),
            View()
                .backPainter(Painter().stroke(2, Colors.invisible.addStyle(color, tileColor)).radius(-1))
                .center()
                .size(18)
                .stylesKeyPath("colorStyles"),
        ])
        .makeView() 
        .wrapInFlexibleContainer();
    };
    
    return ZStack([
        HStack([ 
            CircleColor('0'),
            CircleColor('1'),
            CircleColor('2'),
            CircleColor('3'),
            CircleColor('4'),
            CircleColor('5'),
            CircleColor('6'),
            CircleColor('7'),
            CircleColor('8'), 
        ])
        .spaceViews()
        .makeView() 
        .fillWidth()
        .ignoreAX()
		.keyPath(""), 
    ]).makeView("FGColorPickerView").width(198);
}