markup.push({
			  
	"BoardWindow": {
		nsClass: "Farrago.FGBoardWindow",
		gxWindowTitle: "Farrago",
		gxCanFullScreen: true,
		gxWindowMaxSize: "8000,8000",
		gxWindowMinSize: `${Constants.minWindowWidth}, ${Constants.minWindowHeight}`,
		gxInitialWindowSize: "1100,600",
		gxInitialWindowSizeLarge: "1380,720", 
		gxInitialWindowPosition: "0.5,0.22",
		gxBackColor: Colors.mainBackground,
		gxSavedWindowStateKey: "boardWindow",
		"gxFullSizeContentView": true,
		gxContentView: "BoardWindowContentView",
		"waveformColor": Colors.primaryContent,			// temporary
		dividerColor: Colors.divider,
		"toolbarView" : ToolBar()  
	},

	// --------------------------------------------------------

	"BoardWindowContentView": {
		nsClass: "GXSettingsView",  
		gxWantsLayer: true,
		gxLayout: [ 
			"H:|-0-[trialbar]-0-|",
			`H:|-0-[splitView(>=${Constants.minBoardWidth})]-0-|`, 
			`V:|-(${Constants.headerHeight})-[trialbar]-0-[divider]-0-[splitView(>=${Constants.minBoardHeight})]-0-|`,
			 
		],
		gxChildViews: 
		{  
            "trialbar": TrialBar(),
			"splitView" : View("GXSplitView").childViews([
				SetsPanel(),   
				ZStack([
					
					BoardViewPanel(ShowSetsButton()),
					
					VStack([
						Image("openElsewhere", Colors.tileTintPrimary)
							.size(128),
						Label("This set is open in a secondary window.")
							.fontColor(Colors.tileTintPrimary)
							.fontSize(14)
							.fontWeight(.2),
						Padding(15),
						PushButton("Show Window") 
							.toolTip("Open “<gx>currentTileSet.title</gx>” in seperate window")
							.keyPath("currentTileSet.openInWindowBtn")
							.width(150, 1000)  
							.centerH(),
					]) 
					.makeView("GXSettingsView") 
					.width(300)  
					.viewName("openElsewhere")
					.keyPath("")
					.center(),  
				])
				.makeView()
				.setValues({
					gxHoldingPriority: 50,
					gxFrameRect: "0,0,600,0"
				}),
				InspectorPanel()
			]).setValues({
				gxDividerColor: Colors.secondaryBackground.addStyle("increasedContrast", Colors.contrastBorder),
				gxIntrinsicWidth: -1, 
				gxFrameRect: "0,0,600,0", // Set up the initial width for this panel. 
			}).keyPath("mainSplitViewState"), 
			"divider": Divider(),
		}
	},
			  
	"SetButton" : {
		"+": "ButtonView",
		gxBackPainter: "SetButtonBack",
		gxForePainter: {
			"+": "ControlFore.left.primary.regular.size13",
			gxEdgeInsets: "26, 1, 0, 0",
		},
	},
	"SetButtonBack" : {
		"+": "ImagePainter", 
		gxTintColor: Colors.toolbarButton.showPressed(), 
		gxEdgeInsets: "2, 0, 0, 0",
		gxHAlign: 0,
	}, 

	"TestToolBar" : TemplateView("Toolbar").autoLoadConstraints().fillFrame(),
})

function TileScrollView()
{
	return ScrollView(
		View("Farrago.FGBoardView")
			.viewName("boardView")
			.setValues({
				"gxLockPainter" : ImagePainter("boardLockedPattern").fill(Colors.secondaryContent.withAlpha(0.15)),
				"gxDraggable" : true,
			}) 
			.setAXTitle("Audio Board") 
			.setAXRole("") // Can't think of anything better here? 
			.setAXHelp("Contains audio tiles for the “<gx>currentTileSet.title</gx>“ set.") 
			.childViews([
				// Ghost tiles
				View()
					.viewName("ghostView")
					.fillFrame(), 
				 
				// Empty Set
				VStack([
					View()
						.size(164)
						.centerH()
						.backPainter(ImagePainter("emptySet").tintColor(Colors.tileTintPrimary)),
					ZStack([
						Label("No Sounds Yet")
							.hideKey("tileSet.isSmartSet")
							.fontColor(Colors.tileTintPrimary)
							.fontSize(14)
							.fontWeight(0.4),
						Label("No Matching Sounds")
							.showKey("tileSet.isSmartSet")
							.fontColor(Colors.tileTintPrimary)
							.fontSize(14)
							.fontWeight(0.4),
					]).align("center"),
					Padding(12),
					Label("Drag sound files into this window, or click \n\"Add Sounds...\" from the Sound Set menu") 
						.hideKey("tileSet.isSmartSet")
						.viewName("dragSoundsLabel")
						.fontColor(Colors.tileTintPrimary).fontSize(13),
				])
				.makeView("GXSettingsView") 
				.keyPath("")
				.viewName("emptySetView") 
				.height(230)
				.width(260) 
				.center(),  
			]),
	)
	.autoHidesScrollers(false)
	.setValues({ 
		gxOverlaysScroller: true,
	})
	.vScroller(VScrollbarView())
	// .vScrollWidth(20)
	// .hScroller(HScroller())
	.className("Farrago.FGScrollView")
	.viewName("tilesScrollView") 
	.backColor(Colors.mainBackground);
}


function ListModeFooter()
{
	return ZStack([  
		Divider().topPad(0).fillWidth(),
		HStack([
			
			Padding(16),
			
			PopupButton().setValues({
				gxMenuItems: [
					MenuItem("Queued", 1), 
					MenuItem("Continuous", 2) 
				],
			})
			.addLayout("H:[self(>=110,150@400)]")
			.viewName("playbackMode")
			.label("Playback Mode")
			.keyPath("currentTileSet.playbackMode")
			,  
			
			Padding(),

			Slider()
				.knobPainter(PillKnob().styleByMutating("disabled", (painter) => {
					return painter.inset(1000)
				}))
				.endGap(5) 
				.knobPainterExtra(null)
				.range(0,5)
				.label("Cross-Fade")
				.addLayout("H:[self(>=90,150@400)]")
				.height(22)
				.setAXHelp("Crossfade time in seconds")
				.keyPath("currentTileSet.crossfadeDuration")   
				.readout("currentTileSet.crossfadeReadout")
				.disableKey("currentTileSet.fadeSliderDisabled")
				.wrapInFlexibleContainer()
				.makeView() 
				,
 
			Spacer(),

			Padding(),
			
			PushButton("Reset List")
				.toolTip("Reset performance")
				.width(100)
				.keyPath("currentTileSet.resetPlaybackState")
				.disableKey("currentTileSet.resetButtonDisabled"),
			
			Padding(11), 
		])
		.makeView()
		.centerV(),  
	])
	.makeView("GXSettingsView") 
	.disableKey("currentTileSet.locked")
	.backgroundFill(Colors.mainBackground)
	.viewName("listModeFooter") 
	.height(Constants.listFooterHeight)
	.keyPath("")
}

function SetLockButton()
{
	return ZStack([   
		// Fade animation to and from red
		AnimationView("lock", Colors.warning)  
			.ignoreAX()
			.showKey("currentTileSet.security.setLocked") 
			.keyPath("currentTileSet.security.setLocked") 
			.animateVisibility(1)
			.size(30)
			.topPad(0)
			.centerH(),		
		AnimationView("lock", Colors.secondaryContent)  
			.ignoreAX()
			.hideKey("currentTileSet.security.setLocked") 
			.keyPath("currentTileSet.security.setLocked") 
			.animateVisibility(1)
			.size(30)
			.topPad(0)
			.centerH(),	
		
		// Actual button. Hosted over all the other views to make sure we get the mouse down event
		Button()
			// .size(42)
			.height(42)
			.fillWidth()
			.backPainter(null)
			.forePainter(Painter().fill(Colors.invisible.addStyle("pressed", Colors.mainBackground.withAlpha(0.3))))   
			.toolTip("Lock set", "Unlock set")
			.keyPath("currentTileSet.security.setLocked")
			.checkBoxRole()  
			.onView(Label("Set Locked").fontColor(Colors.warning).ignoreAX().bottomPad(2).centerH().animateVisibility(1., 0.2))
			.offView(Label("Lock Set").fontColor(Colors.secondaryContent).ignoreAX().bottomPad(2).centerH().animateVisibility(1, 0.2)) 
			.fillHeight()
			,   
		
	]).wrapInFlexibleContainer().makeView().wantsLayer();
};

function MainFooterView(leftPaneButton)
{
	return VStack([
		Divider(), 
		HStack([
			View().viewName("spacer1"),

			leftPaneButton.vPad(2).viewName("leftPaneButton"), 

			View().viewName("spacer2"),

			ToolbarButton("icon-add-sound", "Add Sound", "", "newButton") 
				.disableKey(null)
				.setAXTitle("Add Sound")
				.setAXHelp("Select an audio file to add as a new tile.")
				.hideKey("currentTileSet.isSmartSet")
				.viewName("addSoundButton"),
 
			View()
				.viewName("spacer3"),
   
			ListGridToggle() 
				.viewName("layoutButtons")
				.hideKey("currentTileSet.isSmartSet"), 

			View().viewName("spacer4"),
			 
			ToggleInspectorButton("Show Inspector", "Hide Inspector", "showInspectorPanel", asset="show-inspector")  
				.viewName("inspectorToggleButton"), 
			
			View().viewName("spacer5"),
			 
		])
		.spaceViews(["spacer1", 
					 "spacer2",  
					 "spacer3", 
					 "spacer4",
					 "spacer5"]) 
		.makeView(),
	])
	.makeView("GXSettingsView")
    .viewName("mainFooter")
	.keyPath("")
	.height(Constants.footerHeight)
	.backPainter(Painter().fill(Colors.mainBackground));
}

function ListGridToggle() {
    const color = Colors.toolbarButton.showDisabled(Colors.toolbarButton.withAlpha(0.5));

    function createButton(type, iconName, tooltipText, helpText, radii) {
        return ToolbarButtonInternal("", type, "", null, [], null, Colors.toolbarButton)
            .vPad(0)
            .checkBoxRole()
            .width(28) 
            .keyPath(`${type.charAt(0).toLowerCase()}${type.slice(1)}Button`)
            .toolTip(tooltipText)
            .setAXHelp(helpText)
            .offView(
                View()
                    .forePainter(ImagePainter(iconName).tintColor(color).insets(0, 2, 2, 16))
                    .backPainter(Painter().radii(...radii).insets(0, 6, 1, 20).fill(color.withAlpha(0.3)))
            )
            .onView(
                View()
                    .forePainter(ImagePainter(iconName).tintColor(MainThemeColor(Colors.white, Colors.mainBackground)).insets(0, 2, 2, 16))
                    .backPainter(Painter().radii(...radii).insets(0, 6, 1, 20).fill(color))
            )
            .pressView(
                View().forePainter(Painter().fill(Colors.invisible.addStyle("pressed", Colors.mainBackground.withAlpha(0.25))))
            )
            .disableKey("currentTileSet.security.setLocked");
    }

    const gridButton = createButton("Grid", "icon-grid", "Grid layout", "Switch to grid mode", [6, 0, 6, 0]);
    const listButton = createButton("List", "icon-list", "List layout", "Switch to list mode", [0, 6, 0, 6]);

    return HStack([
        Padding(2),
        gridButton,
        listButton,
        Padding(2),
    ])
        .makeView()
        .height(Constants.toolbarButtonHeight)
        .backPainter(Painter().radius(8).addContrast());
}

function SetsPanel(showSets=true)
{
	return VStack([ 

		SeachFieldView().padding2(15, 13, 15, 15),

		VStack([  
			ZStack([ 
				ScrollView(
					VList([ 
						VList([
							View("Farrago.FGTableView") 
								.viewName("searchListView")  
								.leftPad(10)
								.rightPad(0) 
						])  
							.fillWidth()
							.setAXTitle("Search Results")  
							.setAXRole("Search List")
							.showKey("showSearch"),
						(showSets ? SetsList() : View())
							.viewName("setsListSection") 
							.hideKey("showSearch"),  
 
				]))  
				.fillWidth()
				.setValues({
					gxAnimationsDisabled: true, 
				})
				.backColor(Colors.secondaryBackground)
				.vScroller(VScrollbarView(Colors.secondaryBackgroundMix, Colors.secondaryBackground))
				.hScroller(null) 
				.autoHidesScrollers(false)
				.topPad(0),
	
				View("FGSetListDropView")
					.fillFrame()
					.hideKey("searchController.isSearching")
					.backPainter(
						Painter()
							.insets(0,16,0,0)
							.radius(4)
							.stroke(2, Colors.invisible.addStyle("dropping", Colors.tileTintLight)
					)),
	
			])
			.makeView() 
			.fillFrame(),
			 
			Spacer(),
			
			(showSets ? SetsFooterView() : View()),
			
			HStack([
				Padding(12),
				PushButton("Clear Search", false)
					.toolTip("Clear current search")
					.keyPath("searchController.clearSearchField"),
				Padding(10),
			]) 
				.makeView() 
				.height(Constants.footerHeight) 
				.expandKey("showSearch") 
				.fillWidth()
		])
		.makeView("GXSettingsView")	   
		.viewName("setsPanel")  
		.fillFrame(), 
	])
	.makeView() 
	.setValues({
		gxHoldingPriority: 400,
		gxFrameRect: "0,0,210,0", // Set up the initial width for this panel. 
		gxMinimumPanelSize: Constants.minSetsWidth,
		gxCollapsible: true,
	})
	.addLayout(`H:[self(>=${Constants.minSetsWidth})]`)
	.backPainter(Painter().fill(Colors.secondaryBackground).insets(-1, -1, 0, -1));
	
};

function SeachFieldView()
{
	return ZStack([ 
		TextFieldView()	
			.painterByMutating("gxBackPainter", (painter)=> {
				return painter.fill(Colors.mainBackground)
			})
			.painterByMutating("gxTextPainter", (painter)=> {
				return painter.color(MainThemeColor(Colors.black, Colors.white))
			}) 
			.setValues({
				gxEdgeInsets: "25,7,30,0",
				gxPlaceholderString : "Search", 
				gxIsSingleLine: true, 
				gxSelectionColor: Colors.tileTintPrimary.withAlpha(0.3)
			})
			.viewName("searchField") 
			.keyPath("") 
			.setAXHelp("Search audio tiles from all tile sets and sounds available from Freesound") 
			.fillFrame(),

		Button()
			.backPainter(null)
			.forePainter(ImagePainter("xmark.circle")
			.tintColor(Colors.secondaryContent.showPressed()).inset(3))
			.action("clearSearch:") 
			.viewName("clearSearchButton")
			.ignoreAX()
			.size(20)
			.centerV()
			.rightPad(4),

		View()
			.size(12)
			.leftPad(9)
			.topPad(9)
			.forePainter(ImagePainter("magnifying-glass").tintColor(Colors.primaryContent.withAlpha(0.7)).align(0,0.5)),
	])
	.makeView()
	.keyPath("")
	.height(30)
	.fillWidth();
}

function SetsFooterView() {
	return HStack([
		Padding(15),
		ZStack([
			Button()
				.className("GXPopupMenuView")
				.backPainter(TemplatePainter("SetButtonBack.setsPlus").align(0, 0.5))
				.menuItems([
					{ gxTitle: "-" },
					{ gxTitle: "New Set", gxValue: "New Set" },
					{ gxTitle: "New Smart Set", gxValue: "New Smart Set" }
				])
				.keyPath("newSet")
				.ignoreAX()
				.fillFrame(),
			Label("New Set")
				.ignoreAX()
				.fontSize(13)
				.textAlign(1, 0.5)
				.height(20)
				.fillFrame()
		])
			.makeView()
			.width(80)
			.toolTip("Add new set")
			.ignoreAX()
			.viewName("addSetButton"),

		Spacer(),

		Button()
			.size(20)
			.backPainter(TemplatePainter("SetButtonBack.setsMinus").align(0, 0.5))
			.toolTip("Delete selected set")
			.keyPath("deleteSet")
			.ignoreAX()
			.viewName("deleteSetButton"),

		Padding(15),
	])
		.makeView()
		.height(Constants.footerHeight)
		.collapseKey("showSearch");
}

function ToggleInspectorButton(onLabel, offLabel, keyPath, asset, animation=true) 
{
	return HStack([
		Padding(2),
		ZStack([  
			// Lock animation
			AnimationView(asset, Colors.toolbarButton)
				.caarPainter(null,Painter().fill(Colors.toolbarButton).stroke(0, Colors.toolbarButton))
				.ignoreAX()
				.keyPath(keyPath) 
				.size(20)
				.topPad(4)
				.centerH(),		
				
			// Actual button. Hosted over all the other views to make sure we get the mouse down event
			Button() 
				.ignoreAX()  
				.backPainter(null) 
				.forePainter(Painter().fill(Colors.invisible.addStyle("pressed", Colors.mainBackground.withAlpha(0.3))))   
				.toolTip(offLabel, onLabel)
				.keyPath(keyPath)
				.onView(Label(onLabel).textAlign(0,1).fontColor(Colors.toolbarButton).ignoreAX().centerH().vPad(0))
				.vPad(0)
				.offView(Label(offLabel).textAlign(0,1).fontColor(Colors.toolbarButton).ignoreAX().centerH().vPad(0))  
				.vPad(0)
				,   
			
		])
		.makeView() 
		.vPad(0),
		Padding(2),
	])
	.makeView()
	.height(Constants.toolbarButtonHeight)
	.backPainter(Painter().radius(8).addContrast());
};

function SetsList()
{
	return VStack([
		
		HStack([
			Padding(20),
			Label("Sets")
				.ignoreAX()
				.fontSize(14)
				.fontColor(Colors.fontColor)
				.bold(),
			Spacer(),
		]),
		
		VList()
			.fillWidth()
			.ignoreAX()
			.viewName("setsView")
			.setValue(true, "gxAnimationsDisabled")
			.rowSpacing(4)
			.topPad(5),  

		HStack([
			Padding(20),
			Label("Smart Sets") 
				.ignoreAX()
				.viewName("smartSetLabel") 
				.fontSize(14) 
				.fontWeight(0.4),
			Spacer(),
		]).makeView().fillWidth(),
		
		VList()
			.fillWidth()
			.viewName("smartSets")
			.setValue(true, "gxAnimationsDisabled")
			.ignoreAX()
			.topPad(5) 
			.rowSpacing(4),
 

		HStack([
			// hidden AX only buttons
			Button().action("addNewSet:").backPainter(null).size(1).setAXTitle("New Set"),
			Button().action("addSmartSet:").backPainter(null).size(1).setAXTitle("New Smart Set"),
			Button().backPainter(null).size(1).keyPath("deleteSet").setAXTitle("Delete Set"),
		])
		.makeView()
		.ignoreAX(),
		
	])
	.makeView("GXSettingsView") 
	.keyPath("")
	.setAXTitle("Sets List")
	.setAXRole("Sets List Panel");
	
}; 
